/*
 * Copyright (c) 1994 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <DlgUnidraw/dlgviewstate.h>

#include <Dlg/dlgdb.h>
#include <Dlg/dlglib.h>
#include <Dlg/dlgcov.h>
#include <Dlg/dlgfclass.h>
#include <Dlg/dlgfeature.h>

#include <Unidraw/catalog.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/Graphic/graphic.h>

#include <InterViews/transformer.h>

/*****************************************************************************/

Graphic* DlgViewState::_default_gs = nil;
Graphic* DlgViewState::_none_gs = nil;
Graphic* DlgViewState::_wb_gs = nil;
Graphic* DlgViewState::_island_gs = nil;
Graphic* DlgViewState::_ice_gs = nil;
Graphic* DlgViewState::_district_gs = nil;
Graphic* DlgViewState::_city_gs = nil;
Graphic* DlgViewState::_natl_gs = nil;
Graphic* DlgViewState::_ind_gs = nil;
Graphic* DlgViewState::_mil_gs = nil;
Graphic* DlgViewState::_isroad_gs = nil;
Graphic* DlgViewState::_usroad_gs = nil;
Graphic* DlgViewState::_stroad_gs = nil;
Graphic* DlgViewState::_minroad_gs = nil;
Graphic* DlgViewState::_river_gs = nil;
Graphic* DlgViewState::_rr_gs = nil;

DlgViewState::DlgViewState(DlgFeatureClass* fc) : MapViewState() {
    float scale;
    double x1, y1, x2, y2;
    int zone;
    fc->extent(x1, y1, x2, y2, zone);
    if (((DlgDatabase*)fc->coverage()->library()->database())->Is100K())
	scale = 1.0/30.0;
    else
	scale = 1.0/2000.0;
    Transformer* trans = new Transformer(scale,
					 0.0,
					 0.0,
					 scale,
					 -x1*scale,
					 -y1*scale);
    settrans(trans);
}

Graphic* DlgViewState::themegs(MapFeature* feature) const {
    if (!_none_gs) {
	Catalog* catalog = unidraw->GetCatalog();

	_default_gs = new FullGraphic(stdgraphic);
	_default_gs->SetPattern(catalog->FindNonePattern());

	_none_gs = new FullGraphic(stdgraphic);
	_none_gs->SetPattern(catalog->FindNonePattern());
	_none_gs->SetBrush(catalog->FindNoneBrush());

	_wb_gs = new FullGraphic(stdgraphic);
	_wb_gs->SetPattern(catalog->FindGrayLevel(1.0));
	PSColor* ocean = catalog->FindColor("LightSkyBlue");
	PSColor* shoreline = catalog->FindColor("RoyalBlue");
	_wb_gs->SetColors(shoreline,ocean);

	_island_gs = new FullGraphic(stdgraphic);
	_island_gs->SetPattern(catalog->FindGrayLevel(1.0));
	PSColor* land = catalog->FindColor("PeachPuff");
	_island_gs->SetColors(shoreline,land);

	_ice_gs = new FullGraphic(stdgraphic);
	_ice_gs->SetPattern(catalog->FindGrayLevel(1.0));
	PSColor* snow = catalog->FindColor("snow");
	_ice_gs->SetColors(shoreline,snow);

	_district_gs = new FullGraphic(stdgraphic);
	_district_gs->SetPattern(catalog->FindGrayLevel(0.8));
	PSColor* dgray = catalog->FindColor("DimGrey");
	_district_gs->SetColors(dgray,snow);
	_district_gs->FillBg(0);

	_city_gs = new FullGraphic(stdgraphic);
	_city_gs->SetPattern(catalog->FindGrayLevel(0.6));
	PSColor* gray = catalog->FindColor("DimGrey");
	_city_gs->SetColors(gray,snow);
	_city_gs->FillBg(0);

	_natl_gs = new FullGraphic(stdgraphic);
	_natl_gs->SetPattern(catalog->FindGrayLevel(0.8));
	PSColor* green = catalog->FindColor("dark green");
	_natl_gs->SetColors(green,snow);
	_natl_gs->FillBg(0);

	_ind_gs = new FullGraphic(stdgraphic);
	_ind_gs->SetPattern(catalog->FindGrayLevel(0.6));
	PSColor* ired = catalog->FindColor("IndianRed");
	_ind_gs->SetColors(ired,snow);
	_ind_gs->FillBg(0);

	_mil_gs = new FullGraphic(stdgraphic);
	_mil_gs->SetPattern(catalog->FindGrayLevel(0.6));
	PSColor* olive = catalog->FindColor("DarkOliveGreen");
	_mil_gs->SetColors(olive,snow);
	_mil_gs->FillBg(0);

	_isroad_gs = new FullGraphic(stdgraphic);
	_isroad_gs->SetPattern(catalog->FindNonePattern());
	PSColor* red = catalog->FindColor("red");
	_isroad_gs->SetColors(red, red);
	PSBrush* thick = catalog->FindBrush(0xffff, 3);
	_isroad_gs->SetBrush(thick);

	_usroad_gs = new FullGraphic(stdgraphic);
	_usroad_gs->SetPattern(catalog->FindNonePattern());
	_usroad_gs->SetColors(red, red);
	PSBrush* med = catalog->FindBrush(0xffff, 2);
	_usroad_gs->SetBrush(med);

	_stroad_gs = new FullGraphic(stdgraphic);
	_stroad_gs->SetPattern(catalog->FindNonePattern());
	_stroad_gs->SetColors(red, red);

	_minroad_gs = new FullGraphic(stdgraphic);
	_minroad_gs->SetPattern(catalog->FindNonePattern());
	PSColor* khaki = catalog->FindColor("goldenrod");
	_minroad_gs->SetColors(khaki, khaki);

	_river_gs = new FullGraphic(stdgraphic);
	_river_gs->SetPattern(catalog->FindNonePattern());
	_river_gs->SetColors(shoreline, shoreline);

	_rr_gs = new FullGraphic(stdgraphic);
	_rr_gs->SetPattern(catalog->FindNonePattern());
	PSColor* sgray = catalog->FindColor("SlateGray");
	_rr_gs->SetColors(sgray, sgray);
	PSBrush* rrbrush = catalog->FindBrush(0xf0f0, 2);
	_rr_gs->SetBrush(rrbrush);
    }

    DlgFeature* dfeature = (DlgFeature*) feature;
    if (dfeature->long_attr("MAJOR_CODE") == 40) {
	if (dfeature->long_attr("MINOR_CODE") == 199)
	    return _island_gs;
	else if (dfeature->long_attr("MINOR_CODE") == 150)
	    return _island_gs;
	else if (dfeature->long_attr("MINOR_CODE") == 110)
	    return _ice_gs;
	else
	    return _wb_gs;
    } else if (dfeature->long_attr("MAJOR_CODE") == 90) {
	if (dfeature->long_attr("MINOR_CODE") == 100)
	    return _district_gs;
	else if (dfeature->long_attr("MINOR_CODE") == 101)
	    return _city_gs;
	else if (dfeature->long_attr("MINOR_CODE") == 103 ||
		 dfeature->long_attr("MINOR_CODE") == 104 ||
		 dfeature->long_attr("MINOR_CODE") == 105 ||
		 dfeature->long_attr("MINOR_CODE") == 106)
	    return _natl_gs;
	else if (dfeature->long_attr("MINOR_CODE") == 107)
	    return _ind_gs;
	else if (dfeature->long_attr("MINOR_CODE") == 108)
	    return _mil_gs;
	else
	    return _default_gs;
#if 0
    } else if (dfeature->long_attr("MAJOR_CODE") == 91) {
    } else if (dfeature->long_attr("MAJOR_CODE") == 92) {
#endif
    } else if (dfeature->long_attr("MAJOR_CODE") == 102) {
	return _isroad_gs;
    } else if (dfeature->long_attr("MAJOR_CODE") == 103) {
	return _usroad_gs;
    } else if (dfeature->long_attr("MAJOR_CODE") == 104) {
	return _stroad_gs;
    } else if (dfeature->long_attr("MAJOR_CODE") == 290) {
	int minor = dfeature->long_attr("MINOR_CODE");
	if (minor >= 3001 && minor <= 4066)
	    return _river_gs;
	else if (minor == 5001 || minor == 5006 || minor == 5009 ||
		 minor == 5010)
	    return _isroad_gs;
	else if (minor == 5002 || minor == 5013 || minor == 5014 ||
		 minor == 5015 || minor == 5016 || minor == 5017 ||
		 minor == 5018 || minor == 5021 || minor == 5022)
	    return _usroad_gs;
	else if (minor == 5019 || minor == 5020 || minor == 5023 ||
		 minor == 5024 || minor == 5028)
	    return _stroad_gs;
	else if (minor == 5031 || minor == 5041 || minor == 5061 ||
		 minor == 5062)
	    return _minroad_gs;
	else if (minor >= 5071 && minor <= 5080)
	    return _rr_gs;
	else
	    return _default_gs;
    } else
	return _default_gs;
}


