/*
 * Copyright 1994, 1995 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Unidraw/catalog.h>
#include <Unidraw/unidraw.h>
#include <Unidraw/Graphic/graphic.h>

#include <Vpf/vpffclass.h>
#include <Vpf/vpffeature.h>
#include <VpfUnidraw/vpfviewstate.h>

#include <InterViews/transformer.h>

#include <limits.h>

/****************************************************************************/

Graphic* VpfViewState::_poocean_gs = nil;
Graphic* VpfViewState::_poland_gs = nil;
Graphic* VpfViewState::_polbnd_gs = nil;
Graphic* VpfViewState::_poice_gs = nil;
Graphic* VpfViewState::_poline_gs = nil;
Graphic* VpfViewState::_pppoint_gs = nil;
Graphic* VpfViewState::_dnarea_gs = nil;
Graphic* VpfViewState::_dnline_gs = nil;
Graphic* VpfViewState::_dnpoint_gs = nil;
Graphic* VpfViewState::_dnnull_gs = nil;
Graphic* VpfViewState::_hyline_gs = nil;
Graphic* VpfViewState::_tileref_gs = nil;
Graphic* VpfViewState::_aepoint_gs = nil;
Graphic* VpfViewState::_clarea_gs = nil;
Graphic* VpfViewState::_clline_gs = nil;
Graphic* VpfViewState::_clpoint_gs = nil;
Graphic* VpfViewState::_dslake_gs = nil;
Graphic* VpfViewState::_dsland_gs = nil;
Graphic* VpfViewState::_hyarea1_gs = nil;
Graphic* VpfViewState::_hyarea2_gs = nil;
Graphic* VpfViewState::_hyarea3_gs = nil;
Graphic* VpfViewState::_hyarea4_gs = nil;
Graphic* VpfViewState::_hyarea5_gs = nil;
Graphic* VpfViewState::_hyarea6_gs = nil;
Graphic* VpfViewState::_hyarea_gs = nil;
Graphic* VpfViewState::_hypoint_gs = nil;
Graphic* VpfViewState::_hsline_gs = nil;
Graphic* VpfViewState::_hspoint_gs = nil;
Graphic* VpfViewState::_lcarea_gs = nil;
Graphic* VpfViewState::_lcpoint_gs = nil;
Graphic* VpfViewState::_ofline_gs = nil;
Graphic* VpfViewState::_ofpoint_gs = nil;
Graphic* VpfViewState::_phline_gs = nil;
Graphic* VpfViewState::_pparea_gs = nil;
Graphic* VpfViewState::_rrline_gs = nil;
Graphic* VpfViewState::_rdline_gs = nil;
Graphic* VpfViewState::_tsline_gs = nil;
Graphic* VpfViewState::_tspoint_gs = nil;
Graphic* VpfViewState::_utline_gs = nil;
Graphic* VpfViewState::_vgrange_gs = nil;
Graphic* VpfViewState::_vgforst_gs = nil;
Graphic* VpfViewState::_vgtundr_gs = nil;
Graphic* VpfViewState::_unkarea_gs = nil;
Graphic* VpfViewState::_unkline_gs = nil;
Graphic* VpfViewState::_unkpoint_gs = nil;
Graphic* VpfViewState::_text_gs = nil;
Graphic* VpfViewState::_none_gs = nil;

VpfViewState::VpfViewState(PJ* p, double xmin, double ymin, double scale)
: MapViewState(p)
{
    UV data;
    data.u = xmin * DEG_TO_RAD;
    data.v = ymin * DEG_TO_RAD;
    data = pj_fwd(data, p);
    int x = round(data.u);
    int y = round(data.v);
    Transformer* trans = new Transformer(scale,
					 0.0,
					 0.0,
					 scale,
					 -x*scale,
					 -y*scale);
    settrans(trans);
}

VpfViewState::VpfViewState() 
: MapViewState()
{
}

VpfViewState::~VpfViewState() {
}

Transformer* VpfViewState::deftrans() const {
    return _trans;
}

Graphic* VpfViewState::themegs(MapFeature* mf) const {
    if (!_poocean_gs) {
	Catalog* catalog = unidraw->GetCatalog();

	_none_gs = new FullGraphic(stdgraphic);
	_none_gs->SetPattern(catalog->FindNonePattern());
	_none_gs->FillBg(false);
	_none_gs->SetBrush(catalog->FindNoneBrush());

	_poocean_gs = new FullGraphic(stdgraphic);
	_poocean_gs->SetPattern(catalog->FindGrayLevel(1.0));
	PSColor* ocean = catalog->FindColor("LightSkyBlue");
	PSColor* black = catalog->FindColor("black");
	_poocean_gs->SetColors(black,ocean);

	_poland_gs = new FullGraphic(stdgraphic);
	_poland_gs->SetPattern(catalog->FindGrayLevel(1.0));
	PSColor* land = catalog->FindColor("PeachPuff");
	_poland_gs->SetColors(black,land);

	_polbnd_gs = new FullGraphic(stdgraphic);
	_polbnd_gs->SetPattern(catalog->FindGrayLevel(1.0));
	PSColor* landb = catalog->FindColor("PeachPuff");
	_polbnd_gs->SetColors(black,landb);

	_poice_gs = new FullGraphic(stdgraphic);
	PSColor* white = catalog->FindColor("white");
	_poice_gs->SetPattern(catalog->FindGrayLevel(0.1));
	_poice_gs->SetColors(black,white);

	_poline_gs = new FullGraphic(stdgraphic);
	_poline_gs->SetPattern(catalog->FindNonePattern());
	_poline_gs->SetColors(black,black);

	_pppoint_gs = new FullGraphic(stdgraphic);
	PSColor* gray = catalog->FindColor("gray66");
	_pppoint_gs->SetColors(black,gray);
	_pppoint_gs->SetPattern(catalog->FindGrayLevel(1.0));

	_dnarea_gs = new FullGraphic(stdgraphic);
	PSColor* lightblue = catalog->FindColor("LightSkyBlue");
	PSColor* royalblue = catalog->FindColor("RoyalBlue");
	_dnarea_gs->SetColors(royalblue, lightblue);
	_dnarea_gs->SetPattern(catalog->FindGrayLevel(1.0));

	_dnline_gs = new FullGraphic(stdgraphic);
	_dnline_gs->SetColors(royalblue, royalblue);
	_dnline_gs->SetPattern(catalog->FindNonePattern());

	_dnpoint_gs = new FullGraphic(stdgraphic);
	_dnpoint_gs->SetColors(royalblue, royalblue);
	_dnpoint_gs->SetPattern(catalog->FindGrayLevel(1.0));

	_dnnull_gs = new FullGraphic(stdgraphic);
	_dnnull_gs->SetColors(royalblue, white);
	_dnnull_gs->SetPattern(catalog->FindGrayLevel(0.9));
	_dnnull_gs->FillBg(false);

	_hyline_gs = new FullGraphic(stdgraphic);
	PSColor* khaki = catalog->FindColor("chocolate");
	_hyline_gs->SetColors(khaki, khaki);
	_hyline_gs->SetPattern(catalog->FindNonePattern());

	_tileref_gs = new FullGraphic(stdgraphic);
	_tileref_gs->SetColors(black,black);
	_tileref_gs->SetPattern(catalog->ReadPattern("pattern",7));
	_tileref_gs->FillBg(false);

	_aepoint_gs = new FullGraphic(stdgraphic);
	PSColor* nblue = catalog->FindColor("NavyBlue");
	_aepoint_gs->SetColors(black, nblue);
	_aepoint_gs->SetPattern(catalog->FindGrayLevel(1.0));

	_clarea_gs = new FullGraphic(stdgraphic);
	PSColor* sgray = catalog->FindColor("SlateGray");
	_clarea_gs->SetColors(sgray, white);
	_clarea_gs->SetPattern(catalog->FindGrayLevel(0.7));
	_clarea_gs->FillBg(false);

	_clline_gs = new FullGraphic(stdgraphic);
	_clline_gs->SetColors(sgray, sgray);
	_clline_gs->SetPattern(catalog->FindNonePattern());

	_clpoint_gs = new FullGraphic(stdgraphic);
	_clpoint_gs->SetColors(black, sgray);
	_clpoint_gs->SetPattern(catalog->FindGrayLevel(1.0));

	_dslake_gs = new FullGraphic(stdgraphic);
	_dslake_gs->SetColors(royalblue, lightblue);
	_dslake_gs->SetPattern(catalog->FindGrayLevel(1.0));

	_dsland_gs = new FullGraphic(stdgraphic);
	_dsland_gs->SetColors(black, land);
	_dsland_gs->SetPattern(catalog->FindGrayLevel(1.0));

	PSColor* sdlbrn = catalog->FindColor("SaddleBrown");
	PSColor* dkgldrd = catalog->FindColor("DarkGoldenrod");
	PSColor* gldrd = catalog->FindColor("goldenrod");
	PSColor* gold = catalog->FindColor("gold");
	PSColor* ltgldrd = catalog->FindColor("PaleGoldenrod");
	PSColor* plgldrd = catalog->FindColor("LightGoldenrodYellow");

	_hyarea1_gs = new FullGraphic(stdgraphic);
	_hyarea1_gs->SetColors(sdlbrn, sdlbrn);
	_hyarea1_gs->FillBg(false);
	_hyarea1_gs->SetPattern(catalog->FindGrayLevel(0.5));

	_hyarea2_gs = new FullGraphic(stdgraphic);
	_hyarea2_gs->SetColors(dkgldrd, dkgldrd);
	_hyarea2_gs->FillBg(false);
	_hyarea2_gs->SetPattern(catalog->FindGrayLevel(0.5));

	_hyarea3_gs = new FullGraphic(stdgraphic);
	_hyarea3_gs->SetColors(gldrd, gldrd);
	_hyarea3_gs->FillBg(false);
	_hyarea3_gs->SetPattern(catalog->FindGrayLevel(0.5));

	_hyarea4_gs = new FullGraphic(stdgraphic);
	_hyarea4_gs->SetColors(gold, gold);
	_hyarea4_gs->FillBg(false);
	_hyarea4_gs->SetPattern(catalog->FindGrayLevel(0.5));

	_hyarea5_gs = new FullGraphic(stdgraphic);
	_hyarea5_gs->SetColors(ltgldrd, ltgldrd);
	_hyarea5_gs->FillBg(false);
	_hyarea5_gs->SetPattern(catalog->FindGrayLevel(0.5));

	_hyarea6_gs = new FullGraphic(stdgraphic);
	_hyarea6_gs->SetColors(plgldrd, plgldrd);
	_hyarea6_gs->FillBg(false);
	_hyarea6_gs->SetPattern(catalog->FindGrayLevel(0.5));

	_hyarea_gs = new FullGraphic(stdgraphic);
	_hyarea_gs->SetColors(black, black);
	_hyarea_gs->FillBg(false);
	_hyarea_gs->SetPattern(catalog->ReadPattern("pattern",8));

	_hypoint_gs = new FullGraphic(stdgraphic);
	_hypoint_gs->SetColors(black, khaki);
	_hypoint_gs->SetPattern(catalog->FindGrayLevel(1.0));

	_hsline_gs = new FullGraphic(stdgraphic);
	PSColor* sienna = catalog->FindColor("sienna");
	_hsline_gs->SetColors(sienna, sienna);
	_hsline_gs->SetPattern(catalog->FindNonePattern());

	_hspoint_gs = new FullGraphic(stdgraphic);
	_hspoint_gs->SetColors(white, khaki);
	_hspoint_gs->SetPattern(catalog->FindGrayLevel(1.0));

	_lcarea_gs = new FullGraphic(stdgraphic);
	PSColor* cblue = catalog->FindColor("CadetBlue");
	_lcarea_gs->SetColors(cblue, cblue);
	_lcarea_gs->SetPattern(catalog->FindGrayLevel(0.9));
	_lcarea_gs->FillBg(false);

	_lcpoint_gs = new FullGraphic(stdgraphic);
	_lcpoint_gs->SetColors(black, cblue);
	_lcpoint_gs->SetPattern(catalog->FindGrayLevel(1.0));

	_ofline_gs = new FullGraphic(stdgraphic);
	PSColor* ired = catalog->FindColor("IndianRed");
	_ofline_gs->SetColors(ired, ired);
	_ofline_gs->SetPattern(catalog->FindNonePattern());

	_ofpoint_gs = new FullGraphic(stdgraphic);
	_ofpoint_gs->SetColors(black, ired);
	_ofpoint_gs->SetPattern(catalog->FindGrayLevel(1.0));

	_phline_gs = new FullGraphic(stdgraphic);
	PSColor* dblue = catalog->FindColor("DarkSlateBlue");
	_phline_gs->SetColors(dblue, dblue);
	_phline_gs->SetPattern(catalog->FindNonePattern());

	_pparea_gs = new FullGraphic(stdgraphic);
	_pparea_gs->SetColors(black, gray);
	_pparea_gs->SetPattern(catalog->FindGrayLevel(0.75));
	_pparea_gs->FillBg(false);

	_rrline_gs = new FullGraphic(stdgraphic);
	_rrline_gs->SetColors(gray, gray);
	_rrline_gs->SetPattern(catalog->FindNonePattern());
	_rrline_gs->SetBrush(catalog->ReadBrush("brush", 8));
	_rrline_gs->FillBg(false);

	_rdline_gs = new FullGraphic(stdgraphic);
	PSColor* tred = catalog->FindColor("tomato");
	_rdline_gs->SetColors(tred, tred);
	_rdline_gs->SetPattern(catalog->FindNonePattern());

	_tsline_gs = new FullGraphic(stdgraphic);
	PSColor* mar = catalog->FindColor("maroon");
	_tsline_gs->SetColors(mar, mar);
	_tsline_gs->SetPattern(catalog->FindNonePattern());

	_tspoint_gs = new FullGraphic(stdgraphic);
	_tspoint_gs->SetColors(black, mar);
	_tspoint_gs->SetPattern(catalog->FindGrayLevel(1.0));

	_utline_gs = new FullGraphic(stdgraphic);
	PSColor* dr = catalog->FindColor("DarkOrchid");
	_utline_gs->SetColors(dr, dr);
	_utline_gs->SetPattern(catalog->FindNonePattern());

	_vgforst_gs = new FullGraphic(stdgraphic);
	PSColor* gr = catalog->FindColor("ForestGreen");
	_vgforst_gs->SetColors(gr,gr);
	_vgforst_gs->SetPattern(catalog->FindGrayLevel(0.3));
	_vgforst_gs->FillBg(false);

	_vgrange_gs = new FullGraphic(stdgraphic);
	_vgrange_gs->SetColors(gr,gr);
	_vgrange_gs->SetPattern(catalog->FindGrayLevel(0.6));
	_vgrange_gs->FillBg(false);

	_vgtundr_gs = new FullGraphic(stdgraphic);
	_vgtundr_gs->SetColors(gr,gr);
	_vgtundr_gs->SetPattern(catalog->FindGrayLevel(0.9));
	_vgtundr_gs->FillBg(false);

	_unkarea_gs = new FullGraphic(stdgraphic);
	_unkarea_gs->SetColors(black,black);
	_unkarea_gs->SetPattern(catalog->FindGrayLevel(0.9));
	_unkarea_gs->FillBg(false);

	_unkline_gs = new FullGraphic(stdgraphic);
	_unkline_gs->SetPattern(catalog->FindNonePattern());

	_unkpoint_gs = new FullGraphic(stdgraphic);

	_text_gs = new FullGraphic(stdgraphic);
	_text_gs->SetColors(black, white);
	_text_gs->FillBg(false);
    }

    VpfFeature* vf = (VpfFeature*)mf;
    VpfFeatureClass* vfc = vf->feature_class();
    if (vf->type() == MapFeature::TextType) {
	return _text_gs;
    }
    if (strcmp(vfc->name(), "graticule")==0)
	return _unkline_gs;
    if (strcmp(vfc->name(),"tileref")==0)
	return _none_gs;
    if (strcmp(vfc->name(),"poarea")==0 && vf->long_attr("POPYTYPE")==2) {
	return _poocean_gs;
    }
    else if (strcmp(vfc->name(),"poarea")==0 && vf->long_attr("POPYTYPE")==1) {
	return _poland_gs;
    }
    else if (strcmp(vfc->name(),"polbnd")==0) {
	return _polbnd_gs;
    }
    else if (strcmp(vfc->name(),"poarea")==0 &&
	     (vf->long_attr("POPYTYPE")==3 ||
	      vf->long_attr("POPYTYPE")==4 ||
	      vf->long_attr("POPYTYPE")==5)) {
	return _poice_gs;
    }
    else if (strcmp(vfc->name(),"poline")==0) {
	return _poline_gs;
    }
    else if (strcmp(vfc->name(),"pppoint")==0) {
	return _pppoint_gs;
    }
    else if (strcmp(vfc->name(),"dnarea")==0 &&
	     vf->long_attr("DNPYTYPE")==MISSING_VALUE) {
	return _dnnull_gs;
    }
    else if (strcmp(vfc->name(),"dnarea")==0) {
	return _dnarea_gs;
    }
    else if (strcmp(vfc->name(),"dnline")==0) {
	return _dnline_gs;
    }
    else if (strcmp(vfc->name(),"dnpoint")==0) {
	return _dnpoint_gs;
    }
    else if (strcmp(vfc->name(),"hyline")==0) {
	return _hyline_gs;
    }
    else if (strcmp(vfc->name(),"tileref")==0) {
	return _tileref_gs;
    }
    else if (strcmp(vfc->name(),"aepoint")==0) {
	return _aepoint_gs;
    }
    else if (strcmp(vfc->name(),"clarea")==0) {
	return _clarea_gs;
    }
    else if (strcmp(vfc->name(),"clline")==0) {
	return _clline_gs;
    }
    else if (strcmp(vfc->name(),"clpoint")==0) {
	return _clpoint_gs;
    }
    else if (strcmp(vfc->name(),"dspoint")==0 && vf->long_attr("DSPTTYPE")==1) {
	return _dslake_gs;
    }
    else if (strcmp(vfc->name(),"dspoint")==0 && vf->long_attr("DSPTTYPE")==2) {
	return _dsland_gs;
    }
    else if (strcmp(vfc->name(),"hyarea")==0) {
	switch (vf->long_attr("HYPYTYPE")) {
	case 1: return _hyarea1_gs;
	case 2: return _hyarea2_gs;
	case 3: return _hyarea3_gs;
	case 4: return _hyarea4_gs;
	case 5: return _hyarea5_gs;
	case 6: return _hyarea6_gs;
	default: return _hyarea_gs;
	}
    }
    else if (strcmp(vfc->name(),"hypoint")==0) {
	return _hypoint_gs;
    }
    else if (strcmp(vfc->name(),"hsline")==0) {
	return _hsline_gs;
    }
    else if (strcmp(vfc->name(),"hspoint")==0) {
	return _hspoint_gs;
    }
    else if (strcmp(vfc->name(),"lcarea")==0) {
	return _lcarea_gs;
    }
    else if (strcmp(vfc->name(),"lcpoint")==0) {
	return _lcpoint_gs;
    }
    else if (strcmp(vfc->name(),"ofline")==0) {
	return _ofline_gs;
    }
    else if (strcmp(vfc->name(),"ofpoint")==0) {
	return _ofpoint_gs;
    }
    else if (strcmp(vfc->name(),"phline")==0) {
	return _phline_gs;
    }
    else if (strcmp(vfc->name(),"pparea")==0) {
	return _pparea_gs;
    }
    else if (strcmp(vfc->name(),"rrline")==0) {
	return _rrline_gs;
    }
    else if (strcmp(vfc->name(),"rdline")==0) {
	return _rdline_gs;
    }
    else if (strcmp(vfc->name(),"tsline")==0) {
	return _tsline_gs;
    }
    else if (strcmp(vfc->name(),"tspoint")==0) {
	return _tspoint_gs;
    }
    else if (strcmp(vfc->name(),"utline")==0) {
	return _utline_gs;
    }
    else if (strcmp(vfc->name(),"vgarea")==0) {
	switch(vf->long_attr("VGPYTYPE")) {
	case 31:
	case 32:
	case 33: return _vgrange_gs;
	case 41:
	case 42:
	case 43: return _vgforst_gs;
	case 83: return _vgtundr_gs;
	default: return _unkarea_gs;
	}
    }

    // fall-through cases
    if (vf->type() == MapFeature::LineType) {
	return _unkline_gs;
    }
    else if (vf->type() == MapFeature::AreaType) {
	return _unkarea_gs;
    }
    else
	return _unkpoint_gs;
}
